#ifndef SXEVAL_OPERATIONS_LOGARITHM2_HPP
#define SXEVAL_OPERATIONS_LOGARITHM2_HPP

#include "sxeval/AOperation.hpp"
#include <cmath>


/* DEFINITIONS */

namespace sxeval {
namespace operations {

template <typename T>
class Logarithm2 : public AOperation<T> {
public:
    static constexpr const char* KEY = "log2";
    static constexpr const int ARITY_MIN = 1;
    static constexpr const int ARITY_MAX = 1;

    inline Logarithm2(const std::vector<IInstruction<T>*>& args) :
        AOperation<T>(args) {}

    void execute() override;

    inline std::string toString() const override { return KEY; }

};

} /* namespace operations */
} /* namespace sxeval */


/* IMPLEMENTATIONS */

template <typename T>
constexpr const char* sxeval::operations::Logarithm2<T>::KEY;

template <typename T>
void sxeval::operations::Logarithm2<T>::execute() {
    this->_result = static_cast<T>(std::log2(this->_args.front().get()));
}

#endif /* SXEVAL_OPERATIONS_LOGARITHM2_HPP */
